@@ -1,4 +1,4 @@
-/* $Id: wi.h,v 1.3 2004/02/09 21:20:54 benny Exp $ */
+/* $Id: wi.h 545 2004-02-09 21:20:54Z benny $ */
 /*-
  * Copyright (c) 2003,2004 Benedikt Meurer <benny@xfce.org>
  *
@@ -27,8 +27,6 @@
 #ifndef __WI_H__
 #define __WI_H__
 
-#include <glib.h>
-
 #define WI_MAXSTRLEN  (512)
 
 struct wi_device;
@@ -36,7 +34,8 @@ struct wi_device;
 struct wi_stats
 {
   char  ws_netname[WI_MAXSTRLEN]; /* current SSID */
-  int   ws_quality;               /* current signal quality (percent) */
+  int   ws_quality;               /* current signal quality (percent or dBm) */
+  char  ws_qunit[4];              /* % or dBm ? */
   int   ws_rate;                  /* current rate (Mbps) */
   char  ws_vendor[WI_MAXSTRLEN];  /* device vendor name */
 };
